// EA ML_ver01

#property strict

#include <mt4R.mqh>
#include <common_functions.mqh>

// set this so that it points to your R installation. Do NOT remove the --no-save
string R_command = "C:/Program Files/R/R-3.2.3/bin/i386/Rterm.exe --no-save";
int R_debuglevel = 2;

// --  
input int      EA_Magic = 12345;
input int      timeframe = 1;
input int      time_hour_start = 1;
input int      time_hour_end = 23;
input int      trade_lifetime_minutes = 256;
input int      takeprofit_points = 10000;
input int      stoploss_points = 10000;
//input int      dollars_to_lot = 200;
input double   lot = 0.01;


int init() {

   if(Period()!=PeriodSeconds(PERIOD_M1)/60)
   {
      Alert("   M1!");
      return(INIT_FAILED);
   }

   StartR(R_command, R_debuglevel);
   
   Rx("library(gbm)");
   Rx("load('C:/R_study/fx/big_experiment/trained_fx_machine_eurusd.R')");
   Rx("load('C:/R_study/fx/big_experiment/gray_zone_thresholds_eurusd.R')");
    
	 return(0);
}

int deinit() {

   StopR();
   return(0);

}

 // --    
void OnTick()
{

//        static -  Old_Time.
//     OnTick         .
//    ,  ,     .
   static datetime Old_Time;
   datetime New_Time[1];
   bool IsNewBar = false;

//       New_Time[0]
   int copied = CopyTime(Symbol(), timeframe, 0, 1, New_Time);
   if(copied > 0) // ok,  
     {
      if(Old_Time != New_Time[0]) //     
        {
         IsNewBar = true;   //  
         Old_Time = New_Time[0];   //   
        }
     }
   else
     {
      Alert("  ,   = ", GetLastError());
      ResetLastError();
      return;
     }

// --            
   if(IsNewBar == false){
      return;
     }
     
// ---     
   //if(TimeHour(TimeCurrent()) == 9
   //   && TimeMinute(TimeCurrent()) == 0) {
   //   Print(".");
   //  }
   
// ---     

   int order_count = 0;
   
   if(OrdersTotal() > 0)
   {
      for(int i = OrdersTotal() - 1; i >= 0; i -- )
      {
         bool select_order = OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
         if(OrderMagicNumber() == EA_Magic)
         {
            order_count  ++ ;
            if(OrderType()== OP_BUY)
            {
               if((TimeCurrent() - OrderOpenTime()) / 60 >= trade_lifetime_minutes)
               {
                  bool close_order_buy = OrderClose(OrderTicket(), OrderLots(), NormalizeDouble(Bid, Digits), 3, Blue);
                  order_count --;
               }
            }
            
            if(OrderType()== OP_SELL)
            {
               if((TimeCurrent() - OrderOpenTime()) / 60 >= trade_lifetime_minutes)
               {
                  bool close_order_sell = OrderClose(OrderTicket(), OrderLots(), NormalizeDouble(Ask, Digits), 3, Red);
                  order_count --;
               }
            }
         }
      }
   }

// ---     
   
   if(order_count < 1)
      {
   
   int prediction = 0;
   
   // ---       12  (    )
   double double_array[725];

   for(int i = 0; i <= 724; i ++ ) {
   double_array[i] = NormalizeDouble(iOpen(NULL, timeframe, i), Digits);
   }
   
   // ---    R
   
   //> best_inputs
   // "hour"              "lag_diff_128"      "lag_max_diff_64"   "lag_mean_diff_23"  "minute"            "lag_mean_diff_256" "lag_sd_23"        
   // [8] "lag_diff_256"      "lag_diff_181"      "lag_max_diff_23"
   
    Rv("open_prices", double_array);
    Ri("hour", TimeHour(TimeCurrent()));
    Ri("minute", TimeMinute(TimeCurrent()));
    
   // ---   
    
    Rx("inputs_data_frame <- data.frame()");
    Rx("inputs_data_frame[1, 1] <- hour");
    Rx("inputs_data_frame[1, 2] <- open_prices[1] - open_prices[129]");
    Rx("inputs_data_frame[1, 3] <- open_prices[1] - max(open_prices[1:64])");
    Rx("inputs_data_frame[1, 4] <- open_prices[1] - mean(open_prices[1:23])");
    Rx("inputs_data_frame[1, 5] <- minute");
    Rx("inputs_data_frame[1, 6] <- open_prices[1] - mean(open_prices[1:256])");
    Rx("inputs_data_frame[1, 7] <- sd(open_prices[1:23])");
    Rx("inputs_data_frame[1, 8] <- open_prices[1] - open_prices[257]");
    Rx("inputs_data_frame[1, 9] <- open_prices[1] - open_prices[182]");
    Rx("inputs_data_frame[1, 10] <- open_prices[1] - max(open_prices[1:23])");
    
    Rx("colnames(inputs_data_frame) <- c('hour', 'lag_diff_128', 'lag_max_diff_64', 'lag_mean_diff_23', 'minute', 'lag_mean_diff_256', 'lag_sd_23', 'lag_diff_256', 'lag_diff_181', 'lag_max_diff_23')");
    
    // ---  
    
	 Rx("gbm_predict <- predict(object = gbm_obj, newdata = inputs_data_frame, n.trees = 300)");
	 Rx("prediction <- 0");
	 Rx("if (gbm_predict[[1]] < gray_zone_thresholds[1]){ prediction <- -1 }");//
	 Rx("if (gbm_predict[[1]] > gray_zone_thresholds[2]){ prediction <- 1 }");//
	 
	 prediction = Rgi("prediction");

      //double lot = NormalizeDouble(floor(AccountEquity() / dollars_to_lot) / 100, 2);
      if(prediction == 1) {
         int send_order_buy = OrderSend(Symbol(), OP_BUY, lot, NormalizeDouble(Ask, Digits),
         0, 
         0, 
         0, 
         0, 
         EA_Magic, 
         0, 
         clrBlue);
         return;
      }

      if(prediction == -1) {
         int send_order_sell = OrderSend(Symbol(), OP_SELL, lot, NormalizeDouble(Bid, Digits),
         0, 
         0, 
         0, 
         0, 
         EA_Magic, 
         0, 
         clrRed);
         return;
      }
   }
}
   